/* ============================================================================================================================================= /
 * PROJECT:         Orologio 4digig
 * FILE:            Main.h
 * LAST REV:        1.1
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F2550
 * CREATED:         25/06/2014
 * UPDATED:         06/07/2014
 *
 * DESCRIP:         Settings
 *
 * ============================================================================================================================================= /
 * Change History:
 * Rev.             Date                Changes
 * 1.1              06/07/2014          - Prima release
 * 1.0              25/06/2014          - Inizio sviluppo

/* I/O PIC18F2550 ============================================================================================================================== /
 * RA0: a
 * RA1: b
 * RA2: c
 * RA3: d
 * RA4: e
 * RA5: f
 * RA6: g
 * RB0: SET SWITCH
 * RB1: INC SWITCH
 * RB3: dp
 * RB4: CATODO CIFRA DIDECINE ORE
 * RB5: CATODO CIFRA ORE
 * RB6: CATODO CIFRA DECINE MINUTI
 * RB7: CATODO CIFRA MINUTI

 *
/* ============================================================================================================================================ */

/* CONFIGURATION BITS ========================================================================================================================= */
#pragma config PBADEN   = OFF                                                           // PORTB<4:0> come digital I/O
#pragma config XINST    = OFF                                                           // Set istruzioni estese disabilitato
#pragma config DEBUG    = ON                                                            // Debugger Interno (ICD) abilitato
#pragma config LVP      = OFF                                                           // Low voltage program Disablitata
#pragma config FOSC     = INTOSCIO_EC                                                   // Oscillatore interno (IRCF Osc=8MHz)    RA6??
#pragma config PLLDIV   = 1                                                             // PLL no prescale
#pragma config CPUDIV   = OSC1_PLL2                                                     // No post scale
#pragma config PWRT     = ON                                                            // PWRT Abilitato
#pragma config BOR      = OFF                                                           // BOR Disabilitato
#pragma config MCLRE    = ON                                                            // MCLR Abiltato
#pragma config WDT      = OFF                                                           // Disabilitato e controllato da SWDTEN
#pragma config STVREN   = ON                                                            // Restet da Stak uder/over flow abilitato
#pragma config CPD      = OFF                                                           // Protezione dati in EEPROM Disabilitato
#pragma config WRT0     = OFF                                                           // block 0 non protetto
#pragma config WRT1     = OFF                                                           // block 1 non protetto
#pragma config WRTB     = OFF                                                           // Write protect bootblock (ON/OFF)
#pragma config WRTC     = OFF                                                           // Protezione del registro di Configurazione disabilitata

/* CONFIGURAZIONI =============================================================================================================================== */
#define BLINK_DELAY         300                                                         // 500ms
#define PERSIST             2000                                                        // 2000us  tempo di mantenimento su display da 100 a 2000 modifica luminosit
#define DISPLAY_MUX         10                                                          // 10ms ogni quanto scrivo sui display (> PERIST*4)

#define TMR1_PRELOAD        32770
#define FINE_TUNE           10                                                          // Valore in uS che si somma al ritardo di 1

/* CONSTANTI ==================================================================================================================================== */
#define XTAL_FREQ           8                                                           // definisco la frequenza per Libreria Delay

#define DISPLAY             LATA
#define SET_SW              (unsigned char *)&PORTB, 0b00000001                         // puntatore con valore l'indirizzo di PORTB, maschera bit 1 -> RB0
#define INC_SW              (unsigned char *)&PORTB, 0b00000010                         // puntatore con valore l'indirizzo di PORTA, maschera bit 2 -> RB1
#define DP                  LATBbits.LATB3
#define CATHODE_1           LATBbits.LATB4
#define CATHODE_2           LATBbits.LATB5
#define CATHODE_3           LATBbits.LATB6
#define CATHODE_4           LATBbits.LATB7

#define ON                  1
#define OFF                 0

#define ALL                 0
#define HOURS               1
#define MINUTES             2

/* VALORI MINIMI E MASSIMI ================================================================================================================ */
#define MIN_SEC             0x00                                                        // valori minimi e massimi per ogni dato
#define MAX_SEC             0x59
#define MIN_MIN             0x00
#define MAX_MIN             0x59
#define MIN_HOU             0x00
#define MAX_HOU             0x23

/* ============================================================================================================================================= /
 * FUNCTION:        DefineIO
 * Overview:        Definizione e inizializzazione ingressi e uscite
/* ============================================================================================================================================ */

void Define_IO(void) {
    LATA =      0b00000000;                                                             // Reset Latch uscite
    TRISA =     0b00000000;
    LATB =      0b00000000;                                                             // Reset Latch uscite
    TRISB =     0b00000111;
    LATC =      0b00000000;                                                             // Reset Latch uscite
    TRISC =     0b11111111;
}

/* ============================================================================================================================================= /
 * FUNCTION:        Module_settings
 * Overview:        Configurazioni dei moduli interni
/* ============================================================================================================================================ */

void Module_settings(void) {
    UCON =      0;                                                                      // disabilito USB
    UCFG =      1;

    OSCCONbits.IRCF0 = 1;                                                               // IRCF = 111 = 8 MHz (INTOSC drives clock directly)
    OSCCONbits.IRCF1 = 1;
    OSCCONbits.IRCF2 = 1;

    INTCON2bits.RBPU = 0;                                                               // abilito resistenze pull up su PORTB

    CCP1CONbits.CCP1M0 = 0;                                                             // disabilito tutti i comparatori
    CCP1CONbits.CCP1M1 = 0;
    CCP1CONbits.CCP1M2 = 0;
    CCP1CONbits.CCP1M3 = 0;

    CCP2CONbits.CCP2M0 = 0;
    CCP2CONbits.CCP2M1 = 0;
    CCP2CONbits.CCP2M2 = 0;
    CCP2CONbits.CCP2M3 = 0;

    RCONbits.IPEN = 1;                                                                  // abilito interrupt a 2 priorit
    INTCONbits.GIEH = 1;                                                                // abilito interrupt alta priorit
    INTCONbits.GIEL = 1;                                                                // abilito interrupt bassa priorit
    PIE1bits.TMR1IE = 1;                                                                // abilito interrupt timer 1
    IPR1bits.TMR1IP = 1;                                                                // (alta priorit)
    PIE1bits.TMR2IE = 1;                                                                // abilito interrupt timer 2
    IPR1bits.TMR2IP = 0;                                                                // (bassa priorit)



    // TIMER 1, Configurazioni per avere 1 interrupt ogni 0.5s = (65536-preload)*(1/fosc)*prescaler = (65536-49152)*(1/32768)*1
    // TIMER 1, Configurazioni per avere 1 interrupt ogni 1s = (65536-preload)*(1/fosc)*prescaler = (65536-32768)*(1/32768)*1
    T1CONbits.T1CKPS0 =0;                                                               // prescaler 1:1
    T1CONbits.T1CKPS1 =0;                                                               //

    T1CONbits.RD16 = 1;                                                                 // lettura scrittura 16bit

    T1CONbits.T1OSCEN = 1;                                                              // oscillatore abilitato
    T1CONbits.T1SYNC = 1;                                                               // asincrono
    T1CONbits.TMR1CS = 1;                                                               // clock esterno

    TMR1 = TMR1_PRELOAD;                                                                // preload timer 1

    T1CONbits.TMR1ON = 1;                                                               // timer attivo

    //TIMER 2 onfigurazioni per avere 1 interrupt ogni 1ms = (255-preload)*(4/fosc)*prescaler = (255-131)*(4/8000000)*16
    T2CONbits.T2CKPS0 =1;                                                               // prescaler 1:16
    T2CONbits.T2CKPS1 =1;

    TMR2 = 131;

    T2CONbits.TMR2ON = 1;
}